/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : exceptions.h
Purpose : CMSIS compatible definition of exception handler for AT91SAM3S
          To be included in all sources which implement
          or reference exception/irq handler
--------  END-OF-HEADER  ---------------------------------------------
*/

#ifndef EXCEPTIONS_H
#define EXCEPTIONS_H

#ifdef __cplusplus
  extern "C" {
#endif

/* Define WEAK attribute                                            */
/* All default exception handlers are implemented weak,             */
/* so they can be replaced by the application.                      */
#if defined   ( __CC_ARM   )
    #define WEAK __attribute__ ((weak))
#elif defined ( __ICCARM__ )
    #define WEAK __weak
#elif defined (  __GNUC__  )
    #define WEAK __attribute__ ((weak))
#endif

/********************************************************************/

/****** Undefined (not implemented) interrupts **********************/

extern WEAK void IrqHandlerNotUsed(void);

/****** CM3 internal exceptions *************************************/

extern WEAK void NMI_Handler(void);
extern WEAK void HardFault_Handler(void);
extern WEAK void MemManage_Handler(void);
extern WEAK void BusFault_Handler(void);
extern WEAK void UsageFault_Handler(void);
extern WEAK void SVC_Handler(void);
extern WEAK void DebugMon_Handler(void);
extern WEAK void PendSV_Handler(void);
extern WEAK void SysTick_Handler(void);

/****** Controller specific peripheral interrupts *******************/

extern WEAK void SUPC_IRQHandler(void);
extern WEAK void RSTC_IRQHandler(void);
extern WEAK void RTC_IRQHandler(void);
extern WEAK void RTT_IRQHandler(void);
extern WEAK void WDT_IRQHandler(void);
extern WEAK void PMC_IRQHandler(void);
extern WEAK void EEFC_IRQHandler(void);
extern WEAK void UART0_IRQHandler(void);
extern WEAK void UART1_IRQHandler(void);
extern WEAK void SMC_IRQHandler(void);
extern WEAK void PIOA_IRQHandler(void);
extern WEAK void PIOB_IRQHandler(void);
extern WEAK void PIOC_IRQHandler(void);
extern WEAK void USART0_IRQHandler(void);
extern WEAK void USART1_IRQHandler(void);
extern WEAK void MCI_IRQHandler(void);
extern WEAK void TWI0_IRQHandler(void);
extern WEAK void TWI1_IRQHandler(void);
extern WEAK void SPI_IRQHandler(void);
extern WEAK void SSC_IRQHandler(void);
extern WEAK void TC0_IRQHandler(void);
extern WEAK void TC1_IRQHandler(void);
extern WEAK void TC2_IRQHandler(void);
extern WEAK void TC3_IRQHandler(void);
extern WEAK void TC4_IRQHandler(void);
extern WEAK void TC5_IRQHandler(void);
extern WEAK void ADC_IRQHandler(void);
extern WEAK void DAC_IRQHandler(void);
extern WEAK void PWM_IRQHandler(void);
extern WEAK void CRCCU_IRQHandler(void);
extern WEAK void ACC_IRQHandler(void);
extern WEAK void USBD_IRQHandler(void);

#ifdef __cplusplus
  }
#endif

/********************************************************************/

#endif                                  /* Avoid multiple inclusion */

/*************************** End of file ****************************/
