/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for Freescale Kinetis K40 Kit
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define GPIOC_BASE_ADDR  (0x400FF000)

#define GPIOC_PSOR  (*(volatile unsigned int*)(GPIOC_BASE_ADDR + 0x84))
#define GPIOC_PCOR  (*(volatile unsigned int*)(GPIOC_BASE_ADDR + 0x88))
#define GPIOC_PTOR  (*(volatile unsigned int*)(GPIOC_BASE_ADDR + 0x8C))
#define GPIOC_POER  (*(volatile unsigned int*)(GPIOC_BASE_ADDR + 0x94))

#define PORTC_PCR7  (*(volatile unsigned int*)(0x4004B01C))
#define PORTC_PCR8  (*(volatile unsigned int*)(0x4004B020))

#define SIM_SCGC5   (*(volatile unsigned int*)(0x40048038))

/****** Assignment of LEDs ******************************************/

#define LED0_BIT    (7)  /* LED D2 on SAM3S-EK board, active low  */
#define LED1_BIT    (8)  /* LED D3 on SAM3S-EK board, active low  */

#define LEDMASK_ALL ((1uL << LED0_BIT) | (1uL << LED1_BIT))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  unsigned int reg;

  reg         = SIM_SCGC5;      /* Actual value of system clock gating */
  reg        &= ~(1uL << 11);   /* Disable clock for PORTC             */
  reg        |=  (1uL << 11);   /* Enable clock for PORTC              */
  SIM_SCGC5   = reg;            /* Slow slew rate */

  PORTC_PCR7 |= (1uL << 2)      /* Slow slew rate      */
              | (1uL << 5)      /* Open Drain Enable   */
              | (1uL << 6)      /* High drive strength */
              ;
  PORTC_PCR7 &= ~(0x7uL << 8);  /* b10..b8: Clear Pin MUX selection */
  PORTC_PCR7 |=  (0x1uL << 8);  /* b10..b8: Set Alternativ 1, GPIOn */

  PORTC_PCR8 |= (1uL << 2)      /* Slow slew rate      */
              | (1uL << 5)      /* Open Drain Enable   */
              | (1uL << 6)      /* High drive strength */
              ;
  PORTC_PCR8 &= ~(0x7uL << 8);  /* b10..b8: Clear Pin MUX selection */
  PORTC_PCR8 |=  (0x1uL << 8);  /* b10..b8: Set Alternativ 1, GPIOn */

  GPIOC_PSOR  = LEDMASK_ALL;    /* Initially set output high        */
  GPIOC_POER  = LEDMASK_ALL;    /* Enable LED outputs               */
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  GPIOC_PCOR = (1uL << (Index + LED0_BIT)); /* Set output low */
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  GPIOC_PSOR = (1uL << (Index + LED0_BIT)); /* Set output high */
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  GPIOC_PTOR = (1uL << (Index + LED0_BIT)); /* Toggle output   */
}

/****** End Of File *************************************************/
