/*****************************************************************************
 * Copyright (c) 2007 - 2014 Rowley Associates Limited.                      *
 *                                                                           *
 * This file may be distributed under the terms of the License Agreement     *
 * provided with this software.                                              *
 *                                                                           *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE   *
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. *
 *****************************************************************************/

/*****************************************************************************
 *                           Preprocessor Definitions
 *                           ------------------------
 *
 * STARTUP_FROM_RESET
 *
 *   If defined, the program will startup from power-on/reset. If not defined
 *   the program will just loop endlessly from power-on/reset.
 *
 *   This definition is not defined by default on this target because the
 *   debugger is unable to reset this target and maintain control of it over the
 *   JTAG interface. The advantage of doing this is that it allows the debugger
 *   to reset the CPU and run programs from a known reset CPU state on each run.
 *   It also acts as a safety net if you accidently download a program in FLASH
 *   that crashes and prevents the debugger from taking control over JTAG
 *   rendering the target unusable over JTAG. The obvious disadvantage of doing
 *   this is that your application will not startup without the debugger.
 *
 *   We advise that on this target you keep STARTUP_FROM_RESET undefined whilst
 *   you are developing and only define STARTUP_FROM_RESET when development is
 *   complete.
 *
 * __NO_SYSTEM_INIT
 *
 *   If defined, the SystemInit() function will NOT be called. By default SystemInit()
 *   is called after reset to enable the clocks and memories to be initialised
 *   prior to any C startup initialisation.
 *
 * VECTORS_IN_RAM
 *
 *   If defined then the exception vectors are copied from Flash to RAM
 *
 * __TARGET_*
 *
 *  Interrupt vectors table selection
 *
 * __NO_FPU
 *
 *   If defined do NOT turn on the FPU for F4xx/F3xx parts
 *
 * __NO_RUNFAST_MODE
 *
 *   If defined do NOT turn on flush-to-zero and default NaN modes
 *
 *****************************************************************************/

.macro ISR_HANDLER name=
  .section .vectors, "ax"
  .word \name
  .section .init, "ax"
  .thumb_func
  .weak \name
\name:
1: b 1b /* endless loop */
.endm

.macro ISR_RESERVED
  .section .vectors, "ax"
  .word 0
.endm

  .syntax unified
  .global reset_handler

  .section .vectors, "ax"
  .code 16
  .global _vectors

_vectors:
  .word __stack_end__
  .word reset_handler
ISR_HANDLER NMI_Handler
ISR_HANDLER HardFault_Handler
ISR_HANDLER MemManage_Handler
ISR_HANDLER BusFault_Handler
ISR_HANDLER UsageFault_Handler
ISR_RESERVED
ISR_RESERVED
ISR_RESERVED
ISR_RESERVED
ISR_HANDLER SVC_Handler
ISR_HANDLER DebugMon_Handler
ISR_RESERVED
ISR_HANDLER PendSV_Handler
ISR_HANDLER SysTick_Handler
#if defined (STM32F030x6)
  #include "STM32F030x6.vec"
#elif defined (STM32F030x8)
  #include "STM32F030x8.vec"
#elif defined (STM32F031x6)
  #include "STM32F031x6.vec"
#elif defined (STM32F038xx)
  #include "STM32F038xx.vec"
#elif defined (STM32F042x6)
  #include "STM32F042x6.vec"
#elif defined (STM32F048xx)
  #include "STM32F048xx.vec"
#elif defined (STM32F051x8)
  #include "STM32F051x8.vec"
#elif defined (STM32F058xx)
  #include "STM32F058xx.vec"
#elif defined (STM32F071xB)
  #include "STM32F071xB.vec"
#elif defined (STM32F072xB)
  #include "STM32F072xB.vec"
#elif defined (STM32F078xx)
  #include "STM32F078xx.vec"
#elif defined(STM32F10X_LD)
  #include "STM32F10X_LD.vec"
#elif defined(STM32F10X_MD)
  #include "STM32F10X_MD.vec"
#elif defined(STM32F10X_HD)
  #include "STM32F10X_HD.vec"
#elif defined(STM32F10X_XL)
  #include "STM32F10X_XL.vec"
#elif defined(STM32F10X_CL)
  #include "STM32F10X_CL.vec"
#elif defined(STM32F10X_LD_VL)
  #include "STM32F10X_LD_VL.vec"
#elif defined(STM32F10X_MD_VL)
  #include "STM32F10X_MD_VL.vec"
#elif defined(STM32F10X_HD_VL)
  #include "STM32F10X_HD_VL.vec"
#elif defined(STM32F205xx)
  #include "STM32F205xx.vec"
#elif defined(STM32F215xx)
  #include "STM32F215xx.vec"
#elif defined(STM32F207xx)
  #include "STM32F207xx.vec"
#elif defined(STM32F217xx)
  #include "STM32F217xx.vec"
#elif defined(STM32F301x8)
  #include "STM32F301x8.vec"
#elif defined(STM32F302x8)
  #include "STM32F302x8.vec"
#elif defined(STM32F302xC)
  #include "STM32F302xC.vec"
#elif defined(STM32F303x8)
  #include "STM32F303x8.vec"
#elif defined(STM32F303xC)
  #include "STM32F303xC.vec"
#elif defined(STM32F318xx)
  #include "STM32F318xx.vec"
#elif defined(STM32F328xx)
  #include "STM32F328xx.vec"
#elif defined(STM32F334x8)
  #include "STM32F334x8.vec"
#elif defined(STM32F358xx)
  #include "STM32F358xx.vec"
#elif defined(STM32F373xC)
  #include "STM32F373xC.vec"
#elif defined(STM32F378xx)
  #include "STM32F378xx.vec"
#elif defined(STM32F401xC)
  #include "STM32F401xC.vec"
#elif defined(STM32F401xE)
  #include "STM32F401xE.vec"
#elif defined(STM32F405xx)
  #include "STM32F405xx.vec"
#elif defined(STM32F407xx)
  #include "STM32F407xx.vec"
#elif defined(STM32F411xE)
  #include "STM32F411xE.vec"
#elif defined(STM32F415xx)
  #include "STM32F415xx.vec"
#elif defined(STM32F417xx)
  #include "STM32F417xx.vec"
#elif defined(STM32F427xx)
  #include "STM32F427xx.vec"
#elif defined(STM32F429xx)
  #include "STM32F429xx.vec"
#elif defined(STM32F437xx)
  #include "STM32F437xx.vec"
#elif defined(STM32F439xx)
  #include "STM32F439xx.vec"
#elif defined(STM32L051xx)
  #include "STM32L051xx.vec"
#elif defined(STM32L052xx)
  #include "STM32L052xx.vec"
#elif defined(STM32L053xx)
  #include "STM32L053xx.vec"
#elif defined(STM32L061xx)
  #include "STM32L061xx.vec"
#elif defined(STM32L062xx)
  #include "STM32L062xx.vec"
#elif defined(STM32L063xx)
  #include "STM32L063xx.vec"
#elif defined(STM32L1XX_MD)
  #include "STM32L1XX_MD.vec"
#elif defined(STM32L1XX_MDP)
  #include "STM32L1XX_MDP.vec"
#elif defined(STM32L1XX_HD) || defined(STM32L1XX_XL)
  #include "STM32L1XX_HD.vec"
#elif defined (STM32W108C8) || defined (STM32W108CB) || defined (STM32W108CC) || defined (STM32W108CZ) || defined (STM32W108HB)
  #include "STM32W108.vec"
#else
  #error target not defined
#endif
  .section .vectors, "ax"
_vectors_end:


#ifdef VECTORS_IN_RAM
  .section .vectors_ram, "ax"
_vectors_ram:
  .space _vectors_end-_vectors, 0
#endif

  .section .init, "ax"
  .thumb_func

reset_handler:

#ifndef __NO_SYSTEM_INIT
  ldr r0, =__RAM_segment_end__
  mov sp, r0
  bl SystemInit
#endif

#ifdef MEMORY_INIT
  ldr r0, =MemoryInit
  blx r0
#endif


#ifdef VECTORS_IN_RAM
  ldr r0, =__vectors_load_start__
  ldr r1, =__vectors_load_end__
  ldr r2, =_vectors_ram
l0:
  cmp r0, r1
  beq l1
  ldr r3, [r0]
  str r3, [r2]
  adds r0, r0, #4
  adds r2, r2, #4
  b l0
l1:
#endif

#if defined(__FPU_PRESENT) && !defined(__SOFTFP__)
  // Enable CP11 and CP10 with CPACR |= (0xf<<20)
  movw r0, 0xED88
  movt r0, 0xE000
  ldr r1, [r0]
  orrs r1, r1, #(0xf << 20)
  str r1, [r0]
#ifndef __NO_RUNFAST_MODE
  nop
  nop
  nop
  vmrs r0, fpscr
  orrs r0, r0, #(0x3 << 24) // FZ and DN
  vmsr fpscr, r0
  // clear the CONTROL.FPCA bit
  mov r0, #0
  msr control, r0
  // FPDSCR similarly
  movw r1, 0xEF3C
  movt r1, 0xE000
  ldr r0, [r1]
  orrs r0, r0, #(0x3 << 24) // FZ and DN
  str r0, [r1]
#endif
#endif

#ifndef __ARM_ARCH_6M__
  /* Configure vector table offset register */
  ldr r0, =0xE000ED08
#ifdef VECTORS_IN_RAM
  ldr r1, =_vectors_ram
#else
  ldr r1, =_vectors
#endif
  str r1, [r0]
#endif

  b _start

#ifndef __NO_SYSTEM_INIT
  .thumb_func
  .weak SystemInit
SystemInit:
  bx lr
#endif
