/*****************************************************************************
 * Copyright (c) 2007 - 2014 Rowley Associates Limited.                      *
 *                                                                           *
 * This file may be distributed under the terms of the License Agreement     *
 * provided with this software.                                              *
 *                                                                           *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE   *
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. *
 *****************************************************************************/

/*****************************************************************************
 *                           Preprocessor Definitions
 *                           ------------------------
 *
 * STARTUP_FROM_RESET
 *
 *   If defined, the program will startup from power-on/reset. If not defined
 *   the program will just loop endlessly from power-on/reset.
 *
 *   This definition is not defined by default on this target because the
 *   debugger is unable to reset this target and maintain control of it over the
 *   JTAG interface. The advantage of doing this is that it allows the debugger
 *   to reset the CPU and run programs from a known reset CPU state on each run.
 *   It also acts as a safety net if you accidently download a program in FLASH
 *   that crashes and prevents the debugger from taking control over JTAG
 *   rendering the target unusable over JTAG. The obvious disadvantage of doing
 *   this is that your application will not startup without the debugger.
 *
 *   We advise that on this target you keep STARTUP_FROM_RESET undefined whilst
 *   you are developing and only define STARTUP_FROM_RESET when development is
 *   complete.
 *
 * __NO_SYSTEM_INIT
 *
 *   If defined, the SystemInit() function will NOT be called. By default SystemInit()
 *   is called after reset to enable the clocks and memories to be initialised
 *   prior to any C startup initialisation.
 *
 * VECTORS_IN_RAM
 *
 *   If defined then the exception vectors are copied from Flash to RAM
 *
 * __TARGET_*
 *
 *  Interrupt vectors table selection
 *
 * __NO_FPU
 *
 *   If defined do NOT turn on the FPU for F4xx/F3xx parts
 *
 * __NO_RUNFAST_MODE
 *
 *   If defined do NOT turn on flush-to-zero and default NaN modes
 *
 *****************************************************************************/

.macro ISR_HANDLER name=
  .section .vectors, "ax"
  .word \name
  .section .init, "ax"
  .thumb_func
  .weak \name
\name:
1: b 1b /* endless loop */
.endm

.macro ISR_RESERVED
  .section .vectors, "ax"
  .word 0
.endm

  .syntax unified
  .global reset_handler

  .section .vectors, "ax"
  .code 16
  .global _vectors

_vectors:
  .word __stack_end__
#ifdef STARTUP_FROM_RESET
  .word reset_handler
#else
  .word reset_wait
#endif /* STARTUP_FROM_RESET */
ISR_HANDLER NMI_Handler
ISR_HANDLER HardFault_Handler
ISR_HANDLER MemManage_Handler
ISR_HANDLER BusFault_Handler
ISR_HANDLER UsageFault_Handler
ISR_RESERVED
ISR_RESERVED
ISR_RESERVED
ISR_RESERVED
ISR_HANDLER SVC_Handler
ISR_HANDLER DebugMon_Handler
ISR_RESERVED
ISR_HANDLER PendSV_Handler
ISR_HANDLER SysTick_Handler
ISR_HANDLER WWDG_IRQHandler
ISR_HANDLER PVD_AVD_IRQHandler
ISR_HANDLER TAMP_STAMP_IRQHandler
ISR_HANDLER RTC_WKUP_IRQHandler
ISR_HANDLER FLASH_IRQHandler
ISR_HANDLER RCC_IRQHandler
ISR_HANDLER EXTI0_IRQHandler
ISR_HANDLER EXTI1_IRQHandler
ISR_HANDLER EXTI2_IRQHandler
ISR_HANDLER EXTI3_IRQHandler
ISR_HANDLER EXTI4_IRQHandler
ISR_HANDLER DMA1_Stream0_IRQHandler
ISR_HANDLER DMA1_Stream1_IRQHandler
ISR_HANDLER DMA1_Stream2_IRQHandler
ISR_HANDLER DMA1_Stream3_IRQHandler
ISR_HANDLER DMA1_Stream4_IRQHandler
ISR_HANDLER DMA1_Stream5_IRQHandler
ISR_HANDLER DMA1_Stream6_IRQHandler
ISR_HANDLER ADC_IRQHandler
ISR_HANDLER FDCAN1_IT0_IRQHandler
ISR_HANDLER FDCAN2_IT0_IRQHandler
ISR_HANDLER FDCAN1_IT1_IRQHandler
ISR_HANDLER FDCAN2_IT1_IRQHandler
ISR_HANDLER EXTI9_5_IRQHandler
ISR_HANDLER TIM1_BRK_IRQHandler
ISR_HANDLER TIM1_UP_IRQHandler
ISR_HANDLER TIM1_TRG_COM_IRQHandler
ISR_HANDLER TIM1_CC_IRQHandler
ISR_HANDLER TIM2_IRQHandler
ISR_HANDLER TIM3_IRQHandler
ISR_HANDLER TIM4_IRQHandler
ISR_HANDLER I2C1_EV_IRQHandler
ISR_HANDLER I2C1_ER_IRQHandler
ISR_HANDLER I2C2_EV_IRQHandler
ISR_HANDLER I2C2_ER_IRQHandler
ISR_HANDLER SPI1_IRQHandler
ISR_HANDLER SPI2_IRQHandler
ISR_HANDLER USART1_IRQHandler
ISR_HANDLER USART2_IRQHandler
ISR_HANDLER USART3_IRQHandler
ISR_HANDLER EXTI15_10_IRQHandler
ISR_HANDLER RTC_Alarm_IRQHandler
ISR_RESERVED
ISR_HANDLER TIM8_BRK_TIM12_IRQHandler
ISR_HANDLER TIM8_UP_TIM13_IRQHandler
ISR_HANDLER TIM8_TRG_COM_TIM14_IRQHandler
ISR_HANDLER TIM8_CC_IRQHandler
ISR_HANDLER DMA1_Stream7_IRQHandler
ISR_HANDLER FMC_IRQHandler
ISR_HANDLER SDMMC1_IRQHandler
ISR_HANDLER TIM5_IRQHandler
ISR_HANDLER SPI3_IRQHandler
ISR_HANDLER UART4_IRQHandler
ISR_HANDLER UART5_IRQHandler
ISR_HANDLER TIM6_DAC_IRQHandler
ISR_HANDLER TIM7_IRQHandler
ISR_HANDLER DMA2_Stream0_IRQHandler
ISR_HANDLER DMA2_Stream1_IRQHandler
ISR_HANDLER DMA2_Stream2_IRQHandler
ISR_HANDLER DMA2_Stream3_IRQHandler
ISR_HANDLER DMA2_Stream4_IRQHandler
ISR_HANDLER ETH_IRQHandler
ISR_HANDLER ETH_WKUP_IRQHandler
ISR_HANDLER FDCAN_CAL_IRQHandler
ISR_RESERVED
ISR_RESERVED
ISR_RESERVED
ISR_RESERVED
ISR_HANDLER DMA2_Stream5_IRQHandler
ISR_HANDLER DMA2_Stream6_IRQHandler
ISR_HANDLER DMA2_Stream7_IRQHandler
ISR_HANDLER USART6_IRQHandler
ISR_HANDLER I2C3_EV_IRQHandler
ISR_HANDLER I2C3_ER_IRQHandler
ISR_HANDLER OTG_HS_EP1_OUT_IRQHandler
ISR_HANDLER OTG_HS_EP1_IN_IRQHandler
ISR_HANDLER OTG_HS_WKUP_IRQHandler
ISR_HANDLER OTG_HS_IRQHandler
ISR_HANDLER DCMI_IRQHandler
ISR_RESERVED
ISR_HANDLER RNG_IRQHandler
ISR_HANDLER FPU_IRQHandler
ISR_HANDLER UART7_IRQHandler
ISR_HANDLER UART8_IRQHandler
ISR_HANDLER SPI4_IRQHandler
ISR_HANDLER SPI5_IRQHandler
ISR_HANDLER SPI6_IRQHandler
ISR_HANDLER SAI1_IRQHandler
ISR_HANDLER LTDC_IRQHandler
ISR_HANDLER LTDC_ER_IRQHandler
ISR_HANDLER DMA2D_IRQHandler
ISR_HANDLER SAI2_IRQHandler
ISR_HANDLER QUADSPI_IRQHandler
ISR_HANDLER LPTIM1_IRQHandler
ISR_HANDLER CEC_IRQHandler
ISR_HANDLER I2C4_EV_IRQHandler
ISR_HANDLER I2C4_ER_IRQHandler
ISR_HANDLER SPDIF_RX_IRQHandler
ISR_HANDLER OTG_FS_EP1_OUT_IRQHandler
ISR_HANDLER OTG_FS_EP1_IN_IRQHandler
ISR_HANDLER OTG_FS_WKUP_IRQHandler
ISR_HANDLER OTG_FS_IRQHandler
ISR_HANDLER DMAMUX1_OVR_IRQHandler
ISR_HANDLER HRTIM1_Master_IRQHandler
ISR_HANDLER HRTIM1_TIMA_IRQHandler
ISR_HANDLER HRTIM1_TIMB_IRQHandler
ISR_HANDLER HRTIM1_TIMC_IRQHandler
ISR_HANDLER HRTIM1_TIMD_IRQHandler
ISR_HANDLER HRTIM1_TIME_IRQHandler
ISR_HANDLER HRTIM1_FLT_IRQHandler
ISR_HANDLER DFSDM1_FLT0_IRQHandler
ISR_HANDLER DFSDM1_FLT1_IRQHandler
ISR_HANDLER DFSDM1_FLT2_IRQHandler
ISR_HANDLER DFSDM1_FLT3_IRQHandler
ISR_HANDLER SAI3_IRQHandler
ISR_HANDLER SWPMI1_IRQHandler
ISR_HANDLER TIM15_IRQHandler
ISR_HANDLER TIM16_IRQHandler
ISR_HANDLER TIM17_IRQHandler
ISR_HANDLER MDIOS_WKUP_IRQHandler
ISR_HANDLER MDIOS_IRQHandler
ISR_HANDLER JPEG_IRQHandler
ISR_HANDLER MDMA_IRQHandler
ISR_RESERVED
ISR_HANDLER SDMMC2_IRQHandler
ISR_HANDLER HSEM1_IRQHandler
ISR_RESERVED
ISR_HANDLER ADC3_IRQHandler
ISR_HANDLER DMAMUX2_OVR_IRQHandler
ISR_HANDLER BDMA_Channel0_IRQHandler
ISR_HANDLER BDMA_Channel1_IRQHandler
ISR_HANDLER BDMA_Channel2_IRQHandler
ISR_HANDLER BDMA_Channel3_IRQHandler
ISR_HANDLER BDMA_Channel4_IRQHandler
ISR_HANDLER BDMA_Channel5_IRQHandler
ISR_HANDLER BDMA_Channel6_IRQHandler
ISR_HANDLER BDMA_Channel7_IRQHandler
ISR_HANDLER COMP_IRQHandler
ISR_HANDLER LPTIM2_IRQHandler
ISR_HANDLER LPTIM3_IRQHandler
ISR_HANDLER LPTIM4_IRQHandler
ISR_HANDLER LPTIM5_IRQHandler
ISR_HANDLER LPUART1_IRQHandler
ISR_RESERVED
ISR_HANDLER CRS_IRQHandler
ISR_HANDLER ECC_IRQHandler
ISR_HANDLER SAI4_IRQHandler
ISR_RESERVED
ISR_RESERVED
ISR_HANDLER WAKEUP_PIN_IRQHandler

  .section .vectors, "ax"
_vectors_end:

#ifdef VECTORS_IN_RAM
  .section .vectors_ram, "ax"
_vectors_ram:
  .space _vectors_end-_vectors, 0
#endif

  .section .init, "ax"
  .thumb_func

reset_handler:

#ifndef __NO_SYSTEM_INIT
  ldr r0, =__RAM_segment_end__
  mov sp, r0
  bl SystemInit
#endif

#ifdef VECTORS_IN_RAM
  ldr r0, =__vectors_load_start__
  ldr r1, =__vectors_load_end__
  ldr r2, =_vectors_ram
l0:
  cmp r0, r1
  beq l1
  ldr r3, [r0]
  str r3, [r2]
  adds r0, r0, #4
  adds r2, r2, #4
  b l0
l1:
#endif

#if !defined(__NO_FPU) && !defined(__SOFTFP__)
  // Enable CP11 and CP10 with CPACR |= (0xf<<20)
  movw r0, 0xED88
  movt r0, 0xE000
  ldr r1, [r0]
  orrs r1, r1, #(0xf << 20)
  str r1, [r0]
#ifndef __NO_RUNFAST_MODE
  isb
  dsb
  vmrs r0, fpscr
  orrs r0, r0, #(0x3 << 24) // FZ and DN
  vmsr fpscr, r0
  // clear the CONTROL.FPCA bit
  mov r0, #0
  msr control, r0
  // FPDSCR similarly
  movw r1, 0xEF3C
  movt r1, 0xE000
  ldr r0, [r1]
  orrs r0, r0, #(0x3 << 24) // FZ and DN
  str r0, [r1]
#endif
#endif

#ifndef __ARM_ARCH_6M__
  /* Configure vector table offset register */
  ldr r0, =0xE000ED08
#ifdef VECTORS_IN_RAM
  ldr r1, =_vectors_ram
#else
  ldr r1, =_vectors
#endif
  str r1, [r0]
#endif

  b _start

#ifndef __NO_SYSTEM_INIT
  .thumb_func
  .weak SystemInit
SystemInit:
  bx lr
#endif

#ifndef STARTUP_FROM_RESET
  .thumb_func
reset_wait:
1: b 1b /* endless loop */
#endif /* STARTUP_FROM_RESET */
