/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for Energie Micro EFM32-G8XX-STK
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"
#include "em_device.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#define LED0_BIT      (0)    /* USER-LED 0 on STK board, active high  */
#define LED1_BIT      (1)    /* USER-LED 1 on STK board, active high  */
#define LED2_BIT      (2)    /* USER-LED 2 on STK board, active high  */
#define LED3_BIT      (3)    /* USER-LED 3 on STK board, active high  */

#define LEDMASK_ALL   ((1uL << LED0_BIT) | (1uL << LED1_BIT) | (1uL << LED2_BIT) | (1uL << LED3_BIT))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  GPIO_TypeDef *gpio = GPIO;
  uint32_t     gpiomode;
  unsigned int Data;

  /* Enable GPIO */
  CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_GPIO;

  /* Configure GPIO port C 0-3 as LED control outputs */
  gpiomode  = GPIO->P[2].MODEL;
  gpiomode &= ~(_GPIO_P_MODEL_MODE0_MASK |
                _GPIO_P_MODEL_MODE1_MASK |
                _GPIO_P_MODEL_MODE2_MASK |
                _GPIO_P_MODEL_MODE3_MASK);
  gpiomode |= GPIO_P_MODEL_MODE0_PUSHPULL;
  gpiomode |= GPIO_P_MODEL_MODE1_PUSHPULL;
  gpiomode |= GPIO_P_MODEL_MODE2_PUSHPULL;
  gpiomode |= GPIO_P_MODEL_MODE3_PUSHPULL;

  gpio->P[2].MODEL = gpiomode;

  /* All LEDS off */
  Data  = GPIO->P[2].DOUT;
  Data &= ~(LEDMASK_ALL);
  GPIO->P[2].DOUT = Data;
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  GPIO->P[2].DOUT |= (1uL << Index);
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  GPIO->P[2].DOUT &= ~(1uL << Index);
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (GPIO->P[2].DOUT & (1uL << Index)) {
    GPIO->P[2].DOUT &= ~(1uL << Index);
  } else {
    GPIO->P[2].DOUT |=  (1uL << Index);
  }
}

/****** End Of File *************************************************/
