/*****************************************************************************
 * Copyright (c) 2011 Rowley Associates Limited.                             *
 *                                                                           *
 * This file may be distributed under the terms of the License Agreement     *
 * provided with this software.                                              *
 *                                                                           *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE   *
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. *
 *****************************************************************************/

/*****************************************************************************
 *                           Preprocessor Definitions
 *                           ------------------------
 *
 * STARTUP_FROM_RESET
 *
 *   If defined, the program will startup from power-on/reset. If not defined
 *   the program will just loop endlessly from power-on/reset.
 *
 *   This definition is not defined by default on this target because the
 *   debugger is unable to reset this target and maintain control of it over the
 *   JTAG interface. The advantage of doing this is that it allows the debugger
 *   to reset the CPU and run programs from a known reset CPU state on each run.
 *   It also acts as a safety net if you accidently download a program in FLASH
 *   that crashes and prevents the debugger from taking control over JTAG
 *   rendering the target unusable over JTAG. The obvious disadvantage of doing
 *   this is that your application will not startup without the debugger.
 *
 *   We advise that on this target you keep STARTUP_FROM_RESET undefined whilst
 *   you are developing and only define STARTUP_FROM_RESET when development is
 *   complete.
 *
 * NO_SYSTEM_INIT
 *
 *   If defined, the SystemInit() function will NOT be called. By default 
 *   SystemInit() is called after reset to enable the clocks and memories to
 *   be initialised prior to any C startup initialisation.
 *
 *****************************************************************************/

  .global reset_handler

  .syntax unified

  .section .vectors, "ax"
  .code 16
  .align 0
  .global _vectors

.macro DEFAULT_ISR_HANDLER name=
  .thumb_func
  .weak \name
\name:
1: b 1b /* endless loop */
.endm

_vectors:
  .word __stack_end__
  .word reset_handler
  .word NMI_Handler
  .word HardFault_Handler
  .word MemManage_Handler
  .word BusFault_Handler
  .word UsageFault_Handler
  .word 0 // Reserved
  .word 0 // Reserved
  .word 0 // Reserved
  .word 0 // Reserved
  .word SVC_Handler
  .word DebugMon_Handler
  .word 0 // Reserved
  .word PendSV_Handler
  .word SysTick_Handler
  .word DMA_IRQHandler
  .word GPIO_EVEN_IRQHandler
  .word TIMER0_IRQHandler
  .word USART0_RX_IRQHandler
  .word USART0_TX_IRQHandler
  .word USB_IRQHandler
  .word ACMP0_IRQHandler
  .word ADC0_IRQHandler
  .word DAC0_IRQHandler
  .word I2C0_IRQHandler
  .word I2C1_IRQHandler
  .word GPIO_ODD_IRQHandler
  .word TIMER1_IRQHandler
  .word TIMER2_IRQHandler
  .word TIMER3_IRQHandler
  .word USART1_RX_IRQHandler
  .word USART1_TX_IRQHandler
  .word LESENSE_IRQHandler
  .word USART2_RX_IRQHandler
  .word USART2_TX_IRQHandler
  .word UART0_RX_IRQHandler
  .word UART0_TX_IRQHandler
  .word UART1_RX_IRQHandler
  .word UART1_TX_IRQHandler
  .word LEUART0_IRQHandler
  .word LEUART1_IRQHandler
  .word LETIMER0_IRQHandler
  .word PCNT0_IRQHandler
  .word PCNT1_IRQHandler
  .word PCNT2_IRQHandler
  .word RTC_IRQHandler
  .word BURTC_IRQHandler
  .word CMU_IRQHandler
  .word VCMP_IRQHandler
  .word LCD_IRQHandler
  .word MSC_IRQHandler
  .word AES_IRQHandler
  .word EBI_IRQHandler
  .word EMU_IRQHandler

  .section .init, "ax"
  .thumb_func

reset_handler:
#ifndef NO_SYSTEM_INIT
  ldr sp, =__RAM_segment_end__
  ldr r0, =SystemInit
  blx r0
#endif

  /* Configure vector table offset register */
  ldr r0, =0xE000ED08
  ldr r1, =_vectors
  str r1, [r0]

  b _start

DEFAULT_ISR_HANDLER NMI_Handler
DEFAULT_ISR_HANDLER HardFault_Handler
DEFAULT_ISR_HANDLER MemManage_Handler
DEFAULT_ISR_HANDLER BusFault_Handler
DEFAULT_ISR_HANDLER UsageFault_Handler
DEFAULT_ISR_HANDLER SVC_Handler 
DEFAULT_ISR_HANDLER DebugMon_Handler
DEFAULT_ISR_HANDLER PendSV_Handler
DEFAULT_ISR_HANDLER SysTick_Handler 
DEFAULT_ISR_HANDLER DMA_IRQHandler
DEFAULT_ISR_HANDLER GPIO_EVEN_IRQHandler
DEFAULT_ISR_HANDLER TIMER0_IRQHandler
DEFAULT_ISR_HANDLER USART0_RX_IRQHandler
DEFAULT_ISR_HANDLER USART0_TX_IRQHandler
DEFAULT_ISR_HANDLER USB_IRQHandler
DEFAULT_ISR_HANDLER ACMP0_IRQHandler
DEFAULT_ISR_HANDLER ADC0_IRQHandler
DEFAULT_ISR_HANDLER DAC0_IRQHandler
DEFAULT_ISR_HANDLER I2C0_IRQHandler
DEFAULT_ISR_HANDLER I2C1_IRQHandler
DEFAULT_ISR_HANDLER GPIO_ODD_IRQHandler
DEFAULT_ISR_HANDLER TIMER1_IRQHandler
DEFAULT_ISR_HANDLER TIMER2_IRQHandler
DEFAULT_ISR_HANDLER TIMER3_IRQHandler
DEFAULT_ISR_HANDLER USART1_RX_IRQHandler
DEFAULT_ISR_HANDLER USART1_TX_IRQHandler
DEFAULT_ISR_HANDLER LESENSE_IRQHandler
DEFAULT_ISR_HANDLER USART2_RX_IRQHandler
DEFAULT_ISR_HANDLER USART2_TX_IRQHandler
DEFAULT_ISR_HANDLER UART0_RX_IRQHandler
DEFAULT_ISR_HANDLER UART0_TX_IRQHandler
DEFAULT_ISR_HANDLER UART1_RX_IRQHandler
DEFAULT_ISR_HANDLER UART1_TX_IRQHandler
DEFAULT_ISR_HANDLER LEUART0_IRQHandler
DEFAULT_ISR_HANDLER LEUART1_IRQHandler
DEFAULT_ISR_HANDLER LETIMER0_IRQHandler
DEFAULT_ISR_HANDLER PCNT0_IRQHandler
DEFAULT_ISR_HANDLER PCNT1_IRQHandler
DEFAULT_ISR_HANDLER PCNT2_IRQHandler
DEFAULT_ISR_HANDLER RTC_IRQHandler
DEFAULT_ISR_HANDLER BURTC_IRQHandler
DEFAULT_ISR_HANDLER CMU_IRQHandler
DEFAULT_ISR_HANDLER VCMP_IRQHandler
DEFAULT_ISR_HANDLER LCD_IRQHandler
DEFAULT_ISR_HANDLER MSC_IRQHandler
DEFAULT_ISR_HANDLER AES_IRQHandler
DEFAULT_ISR_HANDLER EBI_IRQHandler
DEFAULT_ISR_HANDLER EMU_IRQHandler
